@@ -43,7 +43,7 @@ WATCHDOG_TIMEOUT = 3
 MAX_BRINGING_TO_PAUSED_DURATION = 5
 MAX_SET_STATE_DURATION = 1
 
-DEFAULT_POSITION_LISTENNING_INTERVAL = 500
+DEFAULT_POSITION_LISTENING_INTERVAL = 10
 
 
 class PipelineError(Exception):
@@ -57,7 +57,7 @@ class SimplePipeline(GObject.Object, Loggable):
      - State changes
      - Position seeking
      - Position querying
-     - Along with an periodic callback (optional)
+     - Along with a periodic callback (optional)
 
     Signals:
         state-change: The state of the pipeline changed.
@@ -85,7 +85,7 @@ class SimplePipeline(GObject.Object, Loggable):
         self._bus.add_signal_watch()
         self._bus.connect("message", self._bus_message_cb)
         self._listening = False  # for the position handler
-        self._listening_interval = DEFAULT_POSITION_LISTENNING_INTERVAL
+        self._listening_interval = DEFAULT_POSITION_LISTENING_INTERVAL
         self._listening_sig_id = 0
         self._duration = Gst.CLOCK_TIME_NONE
         # The last known position.
@@ -208,6 +208,8 @@ class SimplePipeline(GObject.Object, Loggable):
         if self.playing():
             self.pause()
         else:
+            if self._duration <= self._last_position:
+                self.simple_seek(0)
             self.play()
 
     # Position and Seeking methods
@@ -225,7 +227,7 @@ class SimplePipeline(GObject.Object, Loggable):
             res, cur = self._pipeline.query_position(Gst.Format.TIME)
         except Exception as e:
             self.handle_exception(e)
-            raise PipelineError("Couldn't get position")
+            raise PipelineError("Couldn't get position") from e
 
         if res:
             self._last_position = cur
@@ -247,7 +249,7 @@ class SimplePipeline(GObject.Object, Loggable):
         self._duration = dur
         return dur
 
-    def activate_position_listener(self, interval=DEFAULT_POSITION_LISTENNING_INTERVAL):
+    def activate_position_listener(self, interval=DEFAULT_POSITION_LISTENING_INTERVAL):
         """Activates the position listener.
 
         When activated, the instance will emit the `position` signal at the
@@ -424,7 +426,7 @@ class SimplePipeline(GObject.Object, Loggable):
             Gst.debug_bin_to_dot_file_with_ts(self._pipeline,
                                               Gst.DebugGraphDetails.ALL,
                                               "pitivi.error")
-            if not self._rendering():
+            if not self.rendering():
                 self._remove_waiting_for_async_done_timeout()
                 self._recover()
         elif message.type == Gst.MessageType.DURATION_CHANGED:
@@ -492,13 +494,13 @@ class SimplePipeline(GObject.Object, Loggable):
             res, dur = self._pipeline.query_duration(Gst.Format.TIME)
         except Exception as e:
             self.handle_exception(e)
-            raise PipelineError("Couldn't get duration: %s" % e)
+            raise PipelineError("Couldn't get duration") from e
 
         if not res:
             raise PipelineError("Couldn't get duration: Returned None")
         return dur
 
-    def _rendering(self):
+    def rendering(self):
         return False
 
 
@@ -550,6 +552,8 @@ class Pipeline(GES.Pipeline, SimplePipeline):
         self._commit_wanted = False
         self._prevent_commits = 0
 
+        self.props.audio_sink = Gst.parse_bin_from_description("level ! audioconvert ! audioresample ! autoaudiosink", True)
+
         if "watchdog" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ''):
             watchdog = Gst.ElementFactory.make("watchdog", None)
             if watchdog:
@@ -561,7 +565,6 @@ class Pipeline(GES.Pipeline, SimplePipeline):
 
     def create_sink(self):
         video_sink, sink_widget = SimplePipeline.create_sink(self)
-        self._pipeline.preview_set_video_sink(video_sink)
         return video_sink, sink_widget
 
     def set_mode(self, mode):
@@ -577,13 +580,12 @@ class Pipeline(GES.Pipeline, SimplePipeline):
 
         return GES.Pipeline.do_change_state(self, state)
 
-    def step_frame(self, framerate, frames_offset):
+    def step_frame(self, frames_offset):
         """Seeks backwards or forwards the specified amount of frames.
 
         This clamps the playhead to the project frames.
 
         Args:
-            framerate (Gst.Fraction): The framerate of the project.
             frames_offset (int): The number of frames to step. Negative number
                 for stepping backwards.
         """
@@ -594,16 +596,11 @@ class Pipeline(GES.Pipeline, SimplePipeline):
                 "Couldn't get position (you're framestepping too quickly), ignoring this request")
             return
 
-        cur_frame = int(
-            round(position * framerate.num / float(Gst.SECOND * framerate.denom), 2))
-        new_frame = cur_frame + frames_offset
-        new_pos = int(new_frame * Gst.SECOND * framerate.denom / framerate.num) + \
-            int((Gst.SECOND * framerate.denom / framerate.num) / 2)
-        Loggable.info(self, "From frame %d to %d at %f fps, seek to %s s",
-                      cur_frame,
-                      new_frame,
-                      framerate.num / framerate.denom,
-                      new_pos / float(Gst.SECOND))
+        cur_frame = self.props.timeline.get_frame_at(position)
+        new_frame = max(0, cur_frame + frames_offset)
+        new_pos = self.props.timeline.get_frame_time(new_frame)
+        self.info("From frame %d to %d - seek to %s",
+                  cur_frame, new_frame, new_pos)
         self.simple_seek(new_pos)
 
     def simple_seek(self, position):
@@ -611,7 +608,7 @@ class Pipeline(GES.Pipeline, SimplePipeline):
             # Nowhere to seek.
             return
 
-        if self._rendering():
+        if self.rendering():
             raise PipelineError("Trying to seek while rendering")
 
         st = Gst.Structure.new_empty("seek")
@@ -675,6 +672,6 @@ class Pipeline(GES.Pipeline, SimplePipeline):
             self._was_empty = True
             self._remove_waiting_for_async_done_timeout()
 
-    def _rendering(self):
+    def rendering(self):
         mask = GES.PipelineFlags.RENDER | GES.PipelineFlags.SMART_RENDER
         return self._pipeline.get_mode() & mask != 0
